@@ -6,7 +6,6 @@
 import logging
 import os
 import sys
-from pathlib import Path
 from logging.handlers import RotatingFileHandler
 
 logger = logging.getLogger("ldf_adapter")  # => This is the key to allow logging from other modules
@@ -54,11 +53,11 @@ def setup_logging():
     from .config import CONFIG
 
     # Ensure log_file exists and is writeable
-    logfile = CONFIG.get("messages", "log_file", fallback="/var/log/feudal/adapter.log")
+    logfile = CONFIG.messages.log_file
     dirname = os.path.dirname(logfile)
 
     loglevel_env = os.environ.get("LOG", "WARNING")
-    loglevel = CONFIG.get("messages", "log_level", fallback=loglevel_env)
+    loglevel = CONFIG.messages.log_level or loglevel_env
     logger.setLevel(loglevel)
 
     if not os.path.isdir(os.path.dirname(logfile)) and os.path.dirname(logfile) != "":
@@ -78,13 +77,13 @@ def setup_logging():
             raise
 
     # Setup logging to logfile
-    file_handler = RotatingFileHandler(logfile, maxBytes=100 ** 6, backupCount=2)
+    file_handler = RotatingFileHandler(logfile, maxBytes=100**6, backupCount=2)
     file_handler.setFormatter(formatter)
     file_handler.setLevel(loglevel)
     logger.addHandler(file_handler)
 
     # Reconsider logging to console:
-    log_to_console = CONFIG.get("messages", "log_to_console", fallback="")
+    log_to_console = CONFIG.messages.log_to_console
     if log_to_console.lower() in ("false", "no"):
         print(f"removing console logger")
         logger.removeHandler(stream_handler)
@@ -98,14 +97,16 @@ def setup_logging():
     else:
         stream_handler.setLevel(loglevel_env)
 
-    logger.debug(f"Running: ----------------------------------------------------------------------------------------------------")
+    logger.debug(
+        f"Running: ----------------------------------------------------------------------------------------------------"
+    )
     logger.debug(f'         {" ".join(sys.argv)} ')
 
     # JSON LOGGER
     # FIXME: jsonlogger name
     jsonlogfile = f"{logfile.rstrip('.log')}-json.log"
     jsonlogger = logging.getLogger("jsondata")
-    jsonfile_handler = RotatingFileHandler(jsonlogfile, maxBytes=100 ** 6, backupCount=2)
+    jsonfile_handler = RotatingFileHandler(jsonlogfile, maxBytes=100**6, backupCount=2)
     jsonfile_handler.setFormatter(formatter)
     jsonfile_handler.setLevel(loglevel)
     jsonlogger.setLevel(loglevel)
